/********************************************************************
filename: 	txtCRenderer.cpp
author:		Morock
created:	27:12:2007 
purpose:	Text render tests
*********************************************************************/

#include "txtCRenderer.h"
#include "..\FastFont9\dhFastFont9.h"

D3DXVECTOR3				gpsvsCRenderer::g_v3Trans;			//Translation vector for WORLD MATRIX computing
D3DXMATRIX				gpsvsCRenderer::g_matCamera;		//Camera matrix for VIEW matrix computing
float					gpsvsCRenderer::g_fAspect;			//Aspect ratio for PROJECTION matrix computing
gpsvsVIRTUALSCREEN_VERTEX gpsvsCRenderer::vsv[4];
D3DXVECTOR2				gpsvsCRenderer::g_ScreenSize;// size of screen in pixels

dhFastFont9	cFont;

//Constructor
gpsvsCRenderer::gpsvsCRenderer()
{

}
//Destructor
gpsvsCRenderer::~gpsvsCRenderer()
{

}

//Set render states
void gpsvsCRenderer::SetupRenderProperties (IDirect3DDevice9* pd3dDevice)
{
	pd3dDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
	pd3dDevice->SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE);
	pd3dDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
}

//BEG - D3DXUtils CALLBACK

	bool	CALLBACK	gpsvsCRenderer::IsDeviceAcceptable( D3DCAPS9* pCaps, D3DFORMAT AdapterFormat, 
															D3DFORMAT BackBufferFormat, bool bWindowed, 
															void* pUserContext )
	{
		return true;
	}
	bool	CALLBACK	gpsvsCRenderer::ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, const D3DCAPS9* pCaps,
															 void* pUserContext )
	{
		return true;
	}
	HRESULT CALLBACK	gpsvsCRenderer::OnCreateDevice( IDirect3DDevice9* pd3dDevice, 
														const D3DSURFACE_DESC* pBackBufferSurfaceDesc,
														void* pUserContext )
	{
		HRESULT hr=S_OK;

		//Initializing view point
		float f_EyePoint_Z=-100.0f;
		float f_EyePoint_Y=0.0f;
		float f_EyePoint_X=0.0f;
		D3DXVECTOR3 vEyePt ( f_EyePoint_X, f_EyePoint_Y, f_EyePoint_Z );
		D3DXVECTOR3 vLookatPt( 0.0f, 0.0f, 1000.0f );
		D3DXVECTOR3 vUpVec ( 0.0f, 1.0f, 0.0f );
		D3DXMATRIX	matView;
		D3DXMatrixLookAtLH(&matView, &vEyePt, &vLookatPt, &vUpVec);
		D3DXMatrixInverse(&g_matCamera,NULL,&matView);
		g_v3Trans = D3DXVECTOR3(0,0,0);
		g_fAspect = 1.0f;

		//Render properties setup
		SetupRenderProperties (pd3dDevice);

		//Test for FastText9
		cFont.Init(pd3dDevice,"font_sl.png","");

		return hr;
	}
	void	CALLBACK	gpsvsCRenderer::OnDestroyDevice( void* pUserContext )
	{
		cFont.Kill();
		return;
	}
	HRESULT CALLBACK	gpsvsCRenderer::OnResetDevice( IDirect3DDevice9* pd3dDevice, 
														const D3DSURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
	{
		HRESULT hr=S_OK;
		//Render properties setup
		SetupRenderProperties (pd3dDevice);
		return hr;
	}
	void	CALLBACK	 gpsvsCRenderer::OnFrameMove( IDirect3DDevice9* pd3dDevice, double fTime, float fElapsedTime, 
													 void* pUserContext )
	{
	}
	void	CALLBACK	 gpsvsCRenderer::OnLostDevice( void* pUserContext )
	{
		return;
	}
	void	CALLBACK	gpsvsCRenderer::OnFrameRender( IDirect3DDevice9* pd3dDevice, double fTime, 
													  float fElapsedTime, void* pUserContext )
	{
		HRESULT hr;

		V( pd3dDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_ARGB(0,140,207,240), 1.0f, 0) );
		if( SUCCEEDED( pd3dDevice->BeginScene() ) )
		{
			SetupRenderProperties(pd3dDevice);

			//Text render
			D3DXMATRIX matRotWorld, matWorld, matRot, matScale;
			D3DXMatrixIdentity(&matWorld);
			D3DXMatrixScaling(&matScale,sin(fTime)*3,sin(fTime)*6,sin(fTime)*2);
			D3DXMatrixRotationZ(&matRot, fTime/2);
			matRotWorld = matRot*matScale;
			cFont.SetWorldTransform(&matRotWorld);
			cFont.Draw("Z-Rotation+Scalling (   )",g_ScreenSize.x/2,g_ScreenSize.y/2,0xFF0023AA);
			D3DXMatrixRotationX(&matRot, -fTime*4);
			matRotWorld = matRot;
			cFont.SetWorldTransform(&matRotWorld);
			cFont.Draw("X-Rotation",(g_ScreenSize.x/2)-50,10,0xFF0023CC);
			D3DXMatrixRotationY(&matRot, -fTime*4);
			matRotWorld = matRot;
			cFont.SetWorldTransform(&matRotWorld);
			cFont.Draw("Y-Rotation",g_ScreenSize.x/2,g_ScreenSize.y-20,0xFFAA0000);
			

		hr = pd3dDevice->EndScene();
		pd3dDevice->Present(NULL,NULL,NULL,NULL);
		}
	}
	LRESULT	CALLBACK	gpsvsCRenderer::MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, 
												bool* pbNoFurtherProcessing, void* pUserContext )
	{
		switch (uMsg)
		{
		case WM_MOUSEMOVE:
			break;
		case WM_SIZE:
			{
				WORD ww,hh;
				ww = LOWORD(lParam);
				hh = HIWORD(lParam);
				g_ScreenSize.x=ww;
				g_ScreenSize.y=hh;
				if(0==hh)
					hh=1;
				g_fAspect = (float)ww/(float)hh;
			}
		}
		return 0;
	}
//END - D3DXUtils CALLBACK

//GRS_VisualSystem Initialization
HRESULT gpsvsCRenderer::INIT(const WCHAR* strWindowTitle, HINSTANCE hInstance, 
							 HICON hIcon, HMENU hMenu,
							 int iWindowX, int iWindowY,
							 int iPixWidth, int iPixHeight)
{
 	// Set the callback functions
 	DXUTSetCallbackDeviceCreated	( gpsvsCRenderer::OnCreateDevice );
 	DXUTSetCallbackDeviceReset		( gpsvsCRenderer::OnResetDevice );
 	DXUTSetCallbackDeviceLost		( &OnLostDevice );
 	DXUTSetCallbackDeviceDestroyed	( &OnDestroyDevice );
 	DXUTSetCallbackMsgProc			( &MsgProc );
 	DXUTSetCallbackFrameRender		( &OnFrameRender );
 	DXUTSetCallbackFrameMove		( &OnFrameMove );

	// Initialize DXUT and create the desired Win32 window and Direct3D device for the application
	DXUTInit( false, false, true, true ); // Parse the command line, handle the default hotkeys, and show msgboxes
	DXUTSetCursorSettings( true, true ); // Show the cursor and clip it when in full screen
	DXUTCreateWindow(strWindowTitle, hInstance,	hIcon, hMenu,
					 iWindowX, iWindowY);
	g_ScreenSize.x=iPixWidth;
	g_ScreenSize.y=iPixHeight;
	DXUTCreateDevice( D3DADAPTER_DEFAULT, true, iPixWidth, iPixHeight, IsDeviceAcceptable, ModifyDeviceSettings );
	// Start the render loop
	DXUTMainLoop();
	return S_OK;
}